import logging
from six.moves.urllib.parse import urlencode
from . import OAuth2Api, SimpleProfile


log = logging.getLogger(__name__)


class YouTube(OAuth2Api):

    def verify_parsed_response(self, data):
        pass

    def whoami(self):
        log.debug("Invoking whoami")
        return self.get("https://www.googleapis.com/oauth2/v2/userinfo")

    def simple_profile(self):
        log.debug("Invoking simple_profile")
        data = self.whoami()

        result = {
            "id": data["id"],
            "name": data["name"],
            "link": data["link"],
            "picture": data["picture"],
        }

        profile = SimpleProfile(**result)
        return profile

    def search(self, query, part='snippet', type='video', **kwargs):
        """https://developers.google.com/youtube/v3/docs/search/list

        Common kwargs to pass to youtube:
        maxResults -- int value
        pageToken -- ex:CAUQAA   this is returned from the previous page result

        the docs link above has a full list of all
        """
        log.debug("Invoking search")

        url = 'https://www.googleapis.com/youtube/v3/search?part={}&q={}'\
            .format(part, query)
        if kwargs:
            additional = urlencode(kwargs)
            url = url + '&{}'.format(additional)
        return self.get(url)

    def user_videos(self, part='snippet', **kwargs):
        """https://developers.google.com/youtube/v3/docs/search/list

        Common kwargs to pass to youtube:
        maxResults -- int value
        pageToken -- ex:CAUQAA   this is returned from the previous page result

        the docs link above has a full list of all
        """

        log.debug("Invoking user_videos")

        url = 'https://www.googleapis.com/youtube/v3/search?part={}&forMine=true&type=video'\
            .format(part)
        if kwargs:
            additional = urlencode(kwargs)
            url = url + '&{}'.format(additional)
        return self.get(url)



import logging
from django.template.response import SimpleTemplateResponse
from django.db import IntegrityError
from chatterbox.models import Activity
from chatterbox.exceptions import (
    RateLimitException, KeyInvalidationException
)


log = logging.getLogger(__name__)


def maybe(func):
    """ Don't use this as a method decorator. Though tempting,
    remember that during method decoration, the class is not created
    yet. `func` will not be bound to anything, and will not have
    `func.im_self` which is important in our use case.
    """
    def action(*args, **kwargs):

        results = None
        instance = func.im_self

        try:
            results = func(*args, **kwargs)
        except RateLimitException:
            log.error("RateLimitException: '%s.%s.%s(*%s, **%s)'",
                      instance.__module__, instance.__class__.__name__,
                      func.im_func.__name__, args, kwargs)
            try:
                instance.key_manager.invalidate_current_key()
                results = func(*args, **kwargs)
            except KeyInvalidationException:
                log.error("KeyInvalidationException: Unable "
                          "to invalidate current key, we are done here")
                results = None

        return results

    return action


def cycle(count):
    current = 0

    while 1:
        yield current
        current = current + 1

        if current >= count:
            current = 0


class KeyManager(object):
    def __init__(self, keys):
        self.keys = keys
        self._cycle = cycle(len(keys))

        self.current_key = keys[0]
        self.current_index = next(self._cycle)

    @property
    def api(self):
        return self.current_key.api

    def invalidate_current_key(self):
        index = next(self._cycle)

        if index == self.current_index:
            log.debug("Unable to invalidate current key. Next key is current key")
            raise KeyInvalidationException

        log.debug("Previous key has been invalidated, switching "
                  "to next available key")

        self.current_key = self.keys[index]
        self.current_index = index


class Collector(object):
    form = None
    template = None
    activity_from_dict = lambda: None

    def __init__(self):
        self.key_manager = None

    @property
    def keys(self):
        try:
            return self.key_manager.keys
        except:
            return []

    @keys.setter
    def keys(self, value):
        self.key_manager = KeyManager(value)

    @property
    def api(self):
        return self.key_manager.api

    def action(self, job):
        raise NotImplementedError("Subclasses must implement the \"action\" method")

    def render(self):
        log.debug("Rendering Collector")
        form = None
        template = self.template

        if self.form:
            log.debug("Initializing Collector Form")
            form = self.form(prefix="data")

        if template and form:
            log.debug("Initializing Collector Template")
            context = {"form": form}
            view = SimpleTemplateResponse(template, context)
            log.debug("Rendering form with template")
            return view.render()

        elif template is None and form:
            log.debug("Rendering form as table")
            return form.as_table()

        else:
            log.debug("No template or form to render")
            return ""

    def create_or_get_activity_from_dict(self, data):
        log.debug("Creating or getting activity from dict: %s", data)
        activity = self.activity_from_dict.__func__(data)
        # try and save the object after setting the ID, if there is error
        # it's because the object is not unique
        try:
            activity.save()
        except IntegrityError:
            # it already exists..but is it from another job?
            log.debug("Activity exists, fetching existing")
            activity = Activity.objects.get(object_id=activity.object_id)

        return activity

    def register_activity(self, activity, job):
        log.debug("Registering activity: %s for job: %s", activity, job)
        count = activity.job.filter(id=job.id).count()

        if count:
            log.error("Job: %s already associated with activity: %s", job, activity)
            raise Exception("Job already associated with activity")

        activity.job.add(job)



import logging
import calendar
from datetime import datetime
from datetime import timedelta
import pytz


log = logging.getLogger(__name__)


def now():
    data = datetime.now(pytz.utc)
    return data - timedelta(microseconds=data.microsecond)


def timestamp_millis_to_datetime(timestamp):
    return datetime.fromtimestamp(float(timestamp / 1000.0), pytz.utc)


def datetime_to_timestamp_millis(obj):
    epoch = datetime.utcfromtimestamp(0).replace(tzinfo=pytz.utc)
    delta = obj - epoch
    return int(delta.total_seconds() * 1000.0)


def datetime_utc_to_pst(obj):
    return obj.astimezone(pytz.timezone("US/Pacific"))


def datetime_to_midnight(obj):
        details = obj.timetuple()
        hours_delta = -details.tm_hour
        minutes_delta = -details.tm_min
        seconds_delta = -details.tm_sec

        offset = timedelta(hours=hours_delta,
                           minutes=minutes_delta,
                           seconds=seconds_delta)

        return obj + offset


def datetime_to_timestamp(obj):
    return calendar.timegm(obj.timetuple())


def string_to_datetime(datetime_string):
    """
    Should be in the form:
    2012-08-29T04:00:00+0000

    This is the inverse of datetime_to_string
    """
    log.debug("Converting string: %s to datetime", datetime_string)
    dt = datetime.strptime(datetime_string, "%Y-%m-%dT%H:%M:%S+0000")
    return dt.replace(tzinfo=pytz.utc)


def datetime_to_string(obj):
    log.debug("Converting datetime to string: %s", obj)
    return obj.strftime("%Y-%m-%dT%H:%M:%S+0000")


def timestamp_to_datetime(timestamp):
    return datetime.fromtimestamp(float(timestamp), pytz.utc)


def aws_date_to_datetime(timestamp):
    """
    2013-04-22T17:24:39.000Z -> datetime()
    """

    log.debug("Converting aws date to datetime: %s", timestamp)
    return activity_stream_date_to_datetime(timestamp)


def twitter_date_to_datetime(timestamp):
    """
    Mon Mar 19 22:10:56 +0000 2012 -> datetime()
    """

    log.debug("Converting twitter date to datetime: %s", timestamp)
    dt = datetime.strptime(timestamp, "%a %b %d %H:%M:%S +0000 %Y")
    return dt.replace(tzinfo=pytz.utc)


def activity_stream_date_to_datetime(timestamp):
    """
    2012-03-19T22:10:56.000Z -> datetime()
    """

    log.debug("Converting activity stream date to datetime: %s", timestamp)
    dt = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S.000Z")
    return dt.replace(tzinfo=pytz.utc)



import logging
from six.moves.urllib.parse import parse_qsl
from .oauth import OAuth2
from .oauth import BEARER_URI


log = logging.getLogger(__name__)


class Facebook(OAuth2):

    # General info about the provider
    provider_url = 'https://facebook.com/'
    docs_url = 'https://developers.facebook.com/docs/'
    category = 'Social'
    api_path = 'chatterbox.api.facebook.Facebook'
    refresh_url = None

    # URLs to interact with the API
    authorize_url = 'https://www.facebook.com/dialog/oauth'
    access_token_url = 'https://graph.facebook.com/oauth/access_token'
    api_domain = 'graph.facebook.com'
    bearer_type = BEARER_URI

    available_permissions = [
        (None, 'read your basic, public information'),  # public_profile
        ('email', 'access your email address'),
        ('read_stream', 'access to read the posts your news feed, or your profile'),
        ('user_about_me', 'access your profile information'),
        ('user_checkins', 'access your checkins'),
        ('user_events', 'access your events'),
        ('user_groups', 'access your groups'),
        ('user_likes', 'access the things you like'),
        ('user_location', 'access your location'),
        ('user_photos', 'access your photos'),
        ('user_status', 'access your most recent status'),
    ]

    def parse_token(self, content):
        data = dict(parse_qsl(content))
        data['expires_in'] = data.get('expires', None)
        return data



import logging
from .oauth import OAuth1


log = logging.getLogger(__name__)


class Twitter(OAuth1):
    # General info about the provider
    provider_url = 'https://www.twitter.com/'
    docs_url = 'https://dev.twitter.com/rest/public'
    category = 'Social'
    api_path = 'chatterbox.api.twitter.Twitter'

    # URLs to interact with the API
    request_token_url = 'https://api.twitter.com/oauth/request_token'
    authorize_url = 'https://api.twitter.com/oauth/authorize'
    access_token_url = 'https://api.twitter.com/oauth/access_token'

    available_permissions = [
        (None, 'read your tweets'),
        ('write', 'read and send tweets'),
        ('dm', 'read and send tweets, including DMs'),
    ]

    # Twitter's permissions model is subtractive, rather than additive. foauth
    # is registered with maximum permissions, which then have to be limited on
    # each authorization call. This mapping converts between the additive model
    # used within foauth and the subtractive model used by Twitter.
    scope_map = {
        None: 'read',
        'write': 'write',
        'dm': None
    }

    def get_request_token_params(self, redirect_uri, scopes):
        params = super(Twitter, self).get_request_token_params(redirect_uri, scopes)

        # Convert to Twitter's permissions model
        scopes = map(lambda x: self.scope_map.get(x or None), scopes)
        if any(scopes):
            params['x_auth_access_type'] = scopes[0]

        return params

    def callback(self, data, *args, **kwargs):
        # if 'denied' in data:
        #     raise OAuthDenied('Denied access to Twitter')

        return super(Twitter, self).callback(data, *args, **kwargs)

    # def get_user_id(self, key):
    #     r = self.api(key, self.api_domains[0], u'/1.1/account/verify_credentials.json')
    #     return unicode(r.json()[u'id'])



import logging
import json
from datetime import (datetime, timedelta)
import pytz
import requests
from oauthlib.oauth1.rfc5849 import SIGNATURE_HMAC, SIGNATURE_TYPE_AUTH_HEADER
from oauthlib.common import generate_token
from requests_oauthlib import OAuth1 as OAuth1Manager
from six.moves.urllib.parse import parse_qsl
from oauthlib.oauth2.rfc6749.clients import base as oauthlib_base


log = logging.getLogger(__name__)


"""Bearer Types

Currently the built in API's use requests-oauthlib, (uses oauthlib under the
hood).

oauthlib's Client supports a property called: `default_token_placement`
requests-oauthlib does not currently support passing this value down to the
`oauthlib.Client`. This is bad news for Services like Instagram who only
accept the token in the URI not in the header. requests-oauthlib will always
send the access token in the header.

`chatterbox.api.Oauth2Api`, will, during it's initialization look back to the
`chatterbox.drivers.*` and pull `bearer_type` now. It will then reach into
the `OAuth2Session._client` and set the `default_token_placement` property.

Acceptable values are derived directly from oauthlib:
https://github.com/idan/oauthlib/blob/master/oauthlib/oauth2/rfc6749/clients/base.py#L23-L25

see this issue on requests-oauthlib:
https://github.com/requests/requests-oauthlib/issues/170
"""

BEARER_HEADER = oauthlib_base.AUTH_HEADER
BEARER_BODY = oauthlib_base.BODY
BEARER_URI = oauthlib_base.URI_QUERY


class OAuthError(Exception):
    pass


class OAuthDenied(Exception):
    pass


class OAuth(object):
    verify = True
    signature_method = SIGNATURE_HMAC
    signature_type = SIGNATURE_TYPE_AUTH_HEADER

    def __init__(self, client_id, client_secret):
        log.debug("Initializing OAuth Driver")

        self.client_id = client_id
        self.client_secret = client_secret
        self.request = None
        self.alias = self.__class__.__name__.lower()

    @property
    def session(self):
        try:
            return self.request.session
        except:
            return {}

    def get_request_token_url(self):
        return self.request_token_url

    def get_access_token_url(self):
        return self.access_token_url

    def get_authorize_url(self, redirect_url, scopes):

        params = self.get_authorize_params(
            redirect_url=redirect_url,
            scopes=scopes,
        )

        req = requests.Request(url=self.authorize_url, params=params)
        return req.prepare().url

    def callback(self, data, redirect_uri):
        """
        Receives the full callback from the service and returns a 2-tuple
        containing the user token and user secret (if applicable).
        """
        raise NotImplementedError("callback() must be defined in a subclass")


class OAuth1(OAuth):
    returns_token = True

    def parse_token(self, content):
        content = dict(parse_qsl(content))

        return {
            'access_token': content['oauth_token'],
            'secret': content['oauth_token_secret'],
        }

    def get_request_token_params(self, redirect_url, scopes):
        return {}

    def get_request_token_response(self, redirect_url, scopes):
        auth = OAuth1Manager(client_key=self.client_id,
                             client_secret=self.client_secret,
                             callback_uri=redirect_url,
                             signature_method=self.signature_method,
                             signature_type=self.signature_type)
        return requests.post(self.get_request_token_url(), auth=auth,
                             params=self.get_request_token_params(redirect_url, scopes),
                             verify=self.verify)

    def get_authorize_params(self, redirect_url, scopes):

        resp = self.get_request_token_response(redirect_url, scopes)
        try:
            data = self.parse_token(resp.content)
        except Exception:
            raise OAuthError('Unable to parse access token')

        self.session["%s_temp_secret" % self.alias] = data['secret']

        if not self.returns_token:
            redirect_url += ('?oauth_token=%s' % data['access_token'])
        return {
            'oauth_token': data['access_token'],
            'oauth_callback': redirect_url,
        }

    def get_access_token_response(self, token, secret, verifier=None):
        auth = OAuth1Manager(client_key=self.client_id,
                             client_secret=self.client_secret,
                             resource_owner_key=token,
                             resource_owner_secret=secret,
                             verifier=verifier,
                             signature_method=self.signature_method,
                             signature_type=self.signature_type)
        return requests.post(self.get_access_token_url(), auth=auth,
                             verify=self.verify)

    def callback(self, data, redirect_uri):
        token = data['oauth_token']
        verifier = data.get('oauth_verifier', None)
        secret = self.session.pop('%s_temp_secret' % self.alias, None)
        resp = self.get_access_token_response(token, secret, verifier)

        try:
            return self.parse_token(resp.content)
        except Exception:
            raise OAuthError('Unable to parse access token')

    def api(self, key, domain, path, method='GET', params=None, data=None,
            headers=None):
        protocol = self.https and 'https' or 'http'
        url = '%s://%s%s' % (protocol, domain, path)
        auth = OAuth1Manager(client_key=self.client_id,
                             client_secret=self.client_secret,
                             resource_owner_key=key.access_token,
                             resource_owner_secret=key.secret,
                             signature_method=self.signature_method,
                             signature_type=self.signature_type)
        return requests.request(method, url, auth=auth, params=params or {},
                                data=data or {}, headers=headers or {},
                                verify=self.verify, stream=True)


class OAuth2(OAuth):
    auth = None
    supports_state = True
    token_type = "Bearer"
    bearer_type = BEARER_HEADER

    def parse_token(self, content):
        data = json.loads(content)
        expires_in = data.get('expires_in', None)

        if expires_in:
            now = datetime.now(pytz.utc)
            data["expires_at"] = now + timedelta(seconds=expires_in)

        return data

    def get_scope_string(self, scopes):
        out = " ".join(scopes)
        return out

    def get_authorize_params(self, redirect_url, scopes):

        state = generate_token()
        self.session['chatterbox_%s_state' % self.alias] = state

        if not self.supports_state:
            redirect_url += ('?state=%s' % state)

        params = {
            'client_id': self.client_id,
            'response_type': 'code',
            'redirect_uri': redirect_url,
            'state': state,
        }

        if any(scopes):
            params['scope'] = self.get_scope_string(scopes)

        return params

    def get_access_token_response(self, redirect_url, data):

        return requests.post(self.get_access_token_url(), {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'authorization_code',
            'code': data.get('code', None),
            'redirect_uri': redirect_url
        }, verify=self.verify, auth=self.auth)

    def callback(self, data, redirect_url):
        state = self.session.pop('chatterbox_%s_state' % self.alias, None)
        # state = flask.session['%s_state' % self.alias]

        if 'state' in data and state != data.get('state', None):
            # potential CSRF
            raise OAuthDenied("invalid state")

        if not self.supports_state:
            redirect_url += ('?state=%s' % state)
        resp = self.get_access_token_response(redirect_url, data)
        return self.parse_token(resp.content)



from django.test import TestCase
from chatterbox.models import Service


class FacebookApiTestCase(TestCase):
    fixtures = ("project/apps/chatterbox/fixtures/users.json",
                "project/apps/chatterbox/fixtures/dump.json")

    def setUp(self):
        self.service = Service.objects.get(key="facebook")
        self.key = self.service.service_keys.first()
        self.api = self.key.api

    def test_user_feed(self):
        results = self.api.user_feed()
        data = results.get("data", None)
        first = data[0]
        self.assertEqual(first.get("from").get("name"), "Dino Petrone")

    def test_brand_feed(self):
        results = self.api.user_feed("11936081183")
        data = results.get("data", None)
        self.assertTrue(len(data) > 0)

    def test_object_detail(self):
        object_id = "10152511415311184"
        result = self.api.object_detail(object_id)

        self.assertTrue(result.get("id", None) == object_id)



import json
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from chatterbox.models import Job


@receiver(post_save, sender=Job)
def job_post_save(sender, **kwargs):
    obj = kwargs["instance"]
    kls = obj.collector.load_driver()
    action = kls()

    hook = getattr(action, "post_save", None)

    if hook:
        hook(obj)


@receiver(post_delete, sender=Job)
def job_post_delete(sender, **kwargs):
    obj = kwargs["instance"]
    kls = obj.collector.load_driver()
    action = kls()

    hook = getattr(action, "post_delete", None)

    if hook:
        hook(obj)





import json
from django.http import JsonResponse
from django.shortcuts import redirect
from oauthlib.common import generate_token
from chatterbox.models import Client


def authorize(request, uuid):
    client = None

    try:
        client = Client.objects.select_related('service').get(uuid=uuid)
    except:
        return JsonResponse({"error": True})

    driver = client.driver
    driver.request = request

    redirect_url = request.GET.get("redirect_url", None)
    scopes = request.GET.getlist("scope[]", None)
    scopes = [x for x in scopes if x != "None"]

    request.session['client_uuid'] = uuid
    url = driver.get_authorize_url(redirect_url, scopes)

    return redirect(url)



import logging
from django import forms
from chatterbox.utils.youtube import activity_from_dict
from . import (Collector, maybe)


log = logging.getLogger(__name__)


class YouTubeSearchForm(forms.Form):
    query = forms.CharField(label='Search Query', max_length=100)


class YouTubeCollector(Collector):
    activity_from_dict = activity_from_dict

    def action(self, job):
        log.debug("Invoking action")

        for status in self.statuses:

            activity = self.create_or_get_activity_from_dict(status)
            try:
                self.register_activity(activity, job)
            except Exception:
                log.debug("Stopping action")
                # need to decide when to stop...
                # at this point this job has already processed this activity
                # before, should we do nothing?  for now we are going to stop
                # all iteration and be done, in the future we might want to
                # keep running the process and run an update action? idk...
                break

    @property
    def statuses(self):
        pass

    def fetch_search_results(self, query, **kwargs):
        return self.api.search(query, **kwargs)

    def fetch_user_videos_results(self, **kwargs):
        return self.api.user_videos(**kwargs)


class YouTubeSearch(YouTubeCollector):
    form = YouTubeSearchForm

    @property
    def statuses(self):
        self.job.data = self.job.data or {}
        query = self.job.data.get('query')
        results = maybe(self.fetch_search_results)(query)
        if results is None:
            raise StopIteration

        messages = results.get('items')

        while 1:
            if len(messages) == 0:
                raise StopIteration

            for message in messages:
                yield message

            log.debug("Fetching next page")

            try:
                page_token = results["nextPageToken"]
            except KeyError:
                raise StopIteration

            if not page_token:
                raise StopIteration

            log.debug("Fetching next page")
            results = maybe(self.fetch_search_results)(query, pageToken=page_token)

            if results is None:
                raise StopIteration

            messages = results.get('items')


class YouTubeUser(YouTubeCollector):

    @property
    def statuses(self):
        results = maybe(self.fetch_user_videos_results)()

        if results is None:
            raise StopIteration

        messages = results.get('items')

        while 1:
            if len(messages) == 0:
                raise StopIteration

            for message in messages:
                yield message

            log.debug("Fetching next page")

            try:
                page_token = results["nextPageToken"]
            except KeyError:
                raise StopIteration

            if not page_token:
                raise StopIteration

            log.debug("Fetching next page")
            results = maybe(self.fetch_user_videos_results)(pageToken=page_token)

            if results is None:
                raise StopIteration

            messages = results.get('items')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields
from django.conf import settings
import chatterbox.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_approved', models.BooleanField(default=chatterbox.models.default_approved)),
                ('published', models.DateTimeField(null=True, blank=True)),
                ('object_id', models.CharField(unique=True, max_length=800)),
                ('object_type', models.CharField(max_length=250, choices=[(b'note', b'Note'), (b'video', b'Video'), (b'image', b'Image')])),
                ('content', models.TextField(null=True, blank=True)),
                ('actor_displayName', models.CharField(max_length=250, null=True, blank=True)),
                ('actor_id', models.CharField(max_length=250, null=True, blank=True)),
                ('provider_displayName', models.CharField(max_length=250, choices=[(b'facebook', b'Facebook'), (b'instagram', b'Instagram'), (b'youtube', b'YouTube'), (b'twitter', b'Twitter')])),
                ('blob', jsonfield.fields.JSONField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Client',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=200)),
                ('uuid', models.CharField(default=chatterbox.models.make_uuid, unique=True, max_length=36, editable=False, db_index=True)),
                ('client_id', models.CharField(max_length=200)),
                ('client_secret', models.CharField(max_length=200)),
                ('redirect_url', models.URLField()),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Collector',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=200)),
                ('driver', models.CharField(unique=True, max_length=200, db_index=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('job_id', models.CharField(default=chatterbox.models.make_uuid, max_length=36, editable=False, db_index=True)),
                ('data', jsonfield.fields.JSONField()),
                ('history', jsonfield.fields.JSONField()),
                ('collector', models.ForeignKey(related_name='collector_actions', to='chatterbox.Collector')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Key',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('access_token', models.CharField(max_length=250)),
                ('secret', models.CharField(max_length=200, null=True, blank=True)),
                ('expires', models.DateTimeField(null=True, blank=True)),
                ('refresh_token', models.CharField(max_length=200, null=True, blank=True)),
                ('service_username', models.CharField(max_length=200)),
                ('service_user_id', models.CharField(max_length=200)),
                ('client', models.ForeignKey(related_name='client_keys', to='chatterbox.Client')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=200)),
                ('key', models.SlugField(unique=True, max_length=200)),
                ('driver', models.CharField(unique=True, max_length=200, db_index=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='key',
            name='service',
            field=models.ForeignKey(related_name='service_keys', to='chatterbox.Service'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='key',
            name='user',
            field=models.ForeignKey(related_name='chatterbox_keys', blank=True, to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='job',
            name='keys',
            field=models.ManyToManyField(related_name='jobs', to='chatterbox.Key'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='collector',
            name='service',
            field=models.ForeignKey(related_name='collectors', to='chatterbox.Service'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='client',
            name='service',
            field=models.ForeignKey(related_name='keys', to='chatterbox.Service'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='activity',
            name='job',
            field=models.ManyToManyField(related_name='job_activity', to='chatterbox.Job'),
            preserve_default=True,
        ),
    ]



from django.contrib import admin
from .models import (Service, Client, Key, Collector, Job, Activity)
from django.utils.html import format_html
from django.conf.urls import url
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_protect
from django.utils.decorators import method_decorator
from django import forms

csrf_protect_m = method_decorator(csrf_protect)


@admin.register(Client)
class ClientAdmin(admin.ModelAdmin):
    change_form_template = "admin/client_change_form.html"


@admin.register(Key)
class KeyAdmin(admin.ModelAdmin):
    list_display = ('get_service', 'get_client', 'user',
                    'get_service_username', 'get_service_user_id')

    def get_service(self, obj):
        return obj.service.label

    def get_client(self, obj):
        return obj.client.label

    def get_service_username(self, obj):
        return obj.service_username

    def get_service_user_id(self, obj):
        return obj.service_user_id

    get_service.short_description = 'Service'
    get_service.admin_order_field = 'service__label'

    get_client.short_description = 'Client'
    get_client.admin_order_field = 'client__label'

    get_service_username.short_description = 'Service User'
    get_service_username.admin_order_field = 'service_username'

    get_service_user_id.short_description = 'Service User ID'
    get_service_user_id.admin_order_field = 'service_user_id'


@admin.register(Service)
class ServiceAdmin(admin.ModelAdmin):
    readonly_fields = ("provider_url", "docs_url")

    def provider_url(self, instance):
        Driver = instance.load_driver()

        if Driver:
            return format_html(
                "<a href=\"{0}\" target=\"_blank\">{0}</a>",
                Driver.provider_url)

        return "Unknown"

    def docs_url(self, instance):
        Driver = instance.load_driver()

        if Driver:
            return format_html(
                "<a href=\"{0}\" target=\"_blank\">{0}</a>",
                Driver.docs_url)

        return "Unknown"

    provider_url.short_description = "Provider Url"
    provider_url.allow_tags = True

    docs_url.short_description = "Docs Url"
    docs_url.allow_tags = True


@admin.register(Collector)
class CollectorAdmin(admin.ModelAdmin):
    pass


def make_approved(modeladmin, request, queryset):
    queryset.update(is_approved=True)
make_approved.short_description = "Mark selected activities as approved"


def make_not_approved(modeladmin, request, queryset):
    queryset.update(is_approved=False)
make_not_approved.short_description = "Mark selected activities as not approved"


@admin.register(Activity)
class ActivityAdmin(admin.ModelAdmin):
    readonly_fields = ('job', 'actor_displayName', 'provider_displayName',
                       'actor_id', 'object_type', 'object_id', 'published',
                       'content')
    list_display = ('object_type', 'admin_media', 'actor_displayName',
                    'provider_displayName', 'published', 'content',
                    'is_approved')

    actions = [make_approved, make_not_approved]


class JobForm(forms.ModelForm):
    service_key = forms.CharField()
    collector = forms.IntegerField()
    keys = forms.ModelMultipleChoiceField(queryset=Key.objects.all())

    class Meta:
        model = Job
        exclude = ("data", "history")

    def _post_clean(self):

        cleaned_data = self.cleaned_data
        extra_data = {}

        collector = Collector.objects.filter(
            service__key=cleaned_data['service_key'],
            id=cleaned_data['collector']).first()

        kls = collector.load_driver()

        if kls.form:
            extra_data = self._process_driver_form(
                kls.form, self.data, self.files)

        obj = self.instance
        obj.collector = collector
        obj.data = extra_data

    def _process_driver_form(self, form, data, files):
        f = form(data=data, files=files, prefix="data")
        result = {}

        if f.is_valid():
            result = f.cleaned_data

        return result


@admin.register(Job)
class JobAdmin(admin.ModelAdmin):
    form = JobForm
    readonly_fields = ("job_id",)
    list_display = ("get_service", "get_collector",
                    "get_service_username", "job_id", "data_json")

    change_form_template = "admin/job_change_form.html"

    def get_service(self, obj):
        return obj.collector.service.label

    def get_collector(self, obj):
        return obj.collector.label

    def get_service_username(self, obj):
        names = obj.keys.all()
        return ', '.join([x.service_username for x in names])

    def save_model(self, request, obj, form, change):
        super(JobAdmin, self).save_model(request, obj, form, change)

        keys = form.cleaned_data['keys']
        obj.keys.add(*keys)

    get_service.short_description = 'Service'
    get_service.admin_order_field = 'service__label'

    get_collector.short_description = 'Collector'
    get_collector.admin_order_field = 'collector__label'

    get_service_username.short_description = 'Service Username(s)'

    def api_services(self, request):
        data = []
        for each in Service.objects.order_by("label").all():
            data.append({"id": each.id, "label": each.label, "key": each.key})

        return JsonResponse(data, safe=False)

    def api_clients(self, request):
        data = []
        service = request.GET.get('service', None)
        collection = []

        if service:
            collection = Client.objects.select_related("service")\
                .filter(service__key=service) \
                .order_by("label")
        else:
            collection = Client.objects.select_related("service")\
                .order_by("label")

        for each in collection:
            data.append({"id": each.id, "label": each.label})

        return JsonResponse(data, safe=False)

    def api_keys(self, request):
        data = []
        collection = []
        service = request.GET.get('service', None)

        if service:
            collection = Key.objects.select_related("service")\
                .filter(service__key=service) \
                .order_by("service_username")
        else:
            collection = Key.objects.select_related("service")\
                .order_by("service_username")

        for each in collection:
            data.append({"id": each.id, "username": each.service_username})

        return JsonResponse(data, safe=False)

    def api_collectors(self, request):
        data = []
        collection = []
        service = request.GET.get('service', None)

        if service:
            collection = Collector.objects.select_related("service")\
                .filter(service__key=service) \
                .order_by("label")
        else:
            collection = Collector.objects.select_related("service")\
                .order_by("label")

        for each in collection:
            data.append({"id": each.id, "label": each.label})

        return JsonResponse(data, safe=False)

    def api_collectors_form(self, request, id):
        html = ""

        try:
            obj = Collector.objects.get(pk=id)
        except:
            obj = None

        if obj:
            kls = obj.load_driver()

            if kls:
                collector = kls()
                html = collector.render()

        return JsonResponse({"form": html})

    def get_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name
        urls = super(JobAdmin, self).get_urls()

        api_urls = [
            url(
                r'^api/services/$',
                self.admin_site.admin_view(self.api_services),
                name='%s_%s_api_services' % info
            ),

            url(
                r'^api/clients/$',
                self.admin_site.admin_view(self.api_clients),
                name='%s_%s_api_clients' % info
            ),

            url(
                r'^api/keys/$',
                self.admin_site.admin_view(self.api_keys),
                name='%s_%s_api_keys' % info
            ),

            url(
                r'^api/collectors/$',
                self.admin_site.admin_view(self.api_collectors),
                name='%s_%s_api_collectors' % info
            ),

            url(
                r'^api/collectors/(?P<id>\d+)/form/$',
                self.admin_site.admin_view(self.api_collectors_form),
                name='%s_%s_api_collectors_form' % info
            ),
        ]

        return api_urls + urls



import json
from django.test import TestCase
from chatterbox.utils.youtube import activity_dict_from_dict
from .utils import load_json


class YoutubeUtils(TestCase):

    def test_activity_dict_from_dict(self):
        data = load_json("youtube-in-basic-video")
        final = load_json("youtube-out-basic-video")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)



import logging
from django import forms
from chatterbox.utils.instagram import activity_from_dict
from . import (Collector, maybe)


log = logging.getLogger(__name__)


class InstagramCollector(Collector):

    def action(self, job):
        log.debug("Invoking action")

        for status in self.statuses:

            activity = self.create_or_get_activity_from_dict(status)
            try:
                self.register_activity(activity, job)
            except Exception:
                log.debug("Stopping action")
                # need to decide when to stop...
                # at this point this job has already processed this activity
                # before, should we do nothing?  for now we are going to stop
                # all iteration and be done, in the future we might want to
                # keep running the process and run an update action? idk...
                break

    @property
    def statuses(self):

        # self.job.data could be '' , or {user_id='foo'} or {query='foo'}
        # so i'm normalizing it here
        self.job.data = self.job.data or {}
        user_id = self.job.data.get("user_id")
        query = self.job.data.get("query")

        results = None

        if query:
            results = maybe(self.fetch_search_results)(query)
        elif user_id:
            results = maybe(self.fetch_user_media)(user_id)
        else:
            results = maybe(self.fetch_user_media)()

        if results is None:
            raise StopIteration

        messages = results.get('data')

        while 1:
            if len(messages) == 0:
                raise StopIteration

            for message in messages:
                yield message

            log.debug("Fetching next page")

            try:
                next_url = results["pagination"]["next_url"]
            except KeyError:
                raise StopIteration

            log.debug("Fetching next page")
            results = maybe(self.fetch_url)(next_url)

            if results is None:
                raise StopIteration

            messages = results.get('data')

    def fetch_search_results(self, query, **kwargs):
        return self.api.search(query=query, **kwargs)

    def fetch_user_media(self, *args, **kwargs):
        return self.api.user_media(*args, **kwargs)

    def fetch_url(self, url, **kwargs):
        return self.api.get(url, **kwargs)


# Wall Collector (this doesn't need a form because it uses the key)
class InstagramWall(InstagramCollector):
    activity_from_dict = activity_from_dict


# Wall Collector + Form
class InstagramUserForm(forms.Form):
    user_id = forms.CharField(label='User ID', max_length=100)


class InstagramWallFromKey(InstagramWall):
    form = InstagramUserForm


# Search Collector + Form
class InstagramSearchForm(forms.Form):
    query = forms.CharField(label='Search Term(s)', max_length=100)


class InstagramSearch(InstagramCollector):
    activity_from_dict = activity_from_dict
    form = InstagramSearchForm



from django.test import TransactionTestCase
from .utils import load_json
import mock
from chatterbox.models import (
    Service, Collector, Job, Activity
)


class CollectorYouTubeWallTestCase(TransactionTestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def setUp(self):
        pass

    def test_basic_search_scrape(self):
        service = Service.objects.get(key='youtube')

        collector = Collector()
        collector.driver = "chatterbox.collectors.youtube.YouTubeSearch"
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.save()
        job.keys.add(key)

        job.data = {"query": "feet"}
        # mock the request response
        return_value = load_json("youtube-search-response")

        with mock.patch('chatterbox.api.youtube.YouTube.search') as mock_search:
            mock_search.return_value = return_value
            job.run()
            self.assertEqual(Activity.objects.all().count(), 5)
            # if we run this command at a later time, with same results, we
            # will not get any duplicate or additional Activity items added
            job.run()
            self.assertEqual(Activity.objects.all().count(), 5)

    def test_basic_user_scrape(self):
        service = Service.objects.get(key='youtube')

        collector = Collector()
        collector.driver = "chatterbox.collectors.youtube.YouTubeUser"
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.save()
        job.keys.add(key)

        # mock the request response
        return_value = load_json("youtube-user-feed-response")

        with mock.patch('chatterbox.api.youtube.YouTube.user_videos') as mock_user_feed:
            mock_user_feed.return_value = return_value
            job.run()
            self.assertEqual(Activity.objects.all().count(), 5)



from chatterbox.collectors import Collector


class DemoCollector(Collector):

    def action(self, job):
        print(job.key.api)

    def post_save(self, job):
        pass

    def post_delete(self, job):
        pass



import json
from django.test import TestCase
from .utils import load_json
from chatterbox.utils.facebook import activity_dict_from_dict


class FacebookUtils(TestCase):

    def test_facebook_link_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-link")
        final = load_json("facebook-out-basic-link")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_facebook_photo_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-photo")
        final = load_json("facebook-out-basic-photo")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_facebook_basic_status_message_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-status+message")
        final = load_json("facebook-out-basic-status+message")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_facebook_basic_status_message_comments_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-status+message+comments")
        final = load_json("facebook-out-basic-status+message+comments")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_facebook_status_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-status")
        final = load_json("facebook-out-basic-status")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_facebook_video_activity_dict_from_dict(self):
        data = load_json("facebook-in-basic-video")
        final = load_json("facebook-out-basic-video")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)



import logging
from ..models import Activity
from .date import activity_stream_date_to_datetime, datetime_to_string


log = logging.getLogger(__name__)


def activity_from_dict(data):
    log.debug("Converting YouTube dict to Activity Model")
    activity_dict = activity_dict_from_dict(data)
    return Activity.from_activity_dict(activity_dict)


def activity_dict_from_dict(blob):
    log.debug("Converting YouTube dict to activity dict: %s", blob)

    stream_object = {}
    stream_object["@context"] = "http://www.w3.org/ns/activitystreams"
    stream_object["@type"] = "Activity"
    date = blob.get("snippet").get("publishedAt")
    date = activity_stream_date_to_datetime(date)
    stream_object["published"] = datetime_to_string(date)

    stream_object["provider"] = {
        "@type": "Service",
        "displayName": "YouTube"
    }
    snippet = blob.get("snippet")
    stream_object["actor"] = {
        "@type": "Person",
        "@id": "https://www.youtube.com/user/{}".format(snippet.get("channelTitle")),
        "displayName": snippet.get("channelTitle"),
    }

    stream_object["object"] = {
        "@id": "https://www.youtube.com/watch?v={}".format(blob.get("id").get("videoId")),
        "@type": "Video",
        "displayName": snippet.get("title"),
        "url": [{
            "href": "https://www.youtube.com/watch?v={}".format(blob.get("id").get("videoId")),
            "@type": "Link"
        }],
        "content": snippet.get("description"),
        "youtube:etag": blob.get("etag"),
        "youtube:kind": blob.get("kind"),
        "youtube:id:kind": blob.get("id").get("kind"),
        "youtube:channelId": snippet.get("channelId"),
        "youtube:liveBroadcastContent": snippet.get("liveBroadcastContent"),
        "image": [
            {
                "@type": "Link",
                "href": snippet.get("thumbnails").get("default").get("url"),
                "mediaType": "image/jpeg",
                "youtube:resolution": "default"
            },
            {
                "@type": "Link",
                "href": snippet.get("thumbnails").get("medium").get("url"),
                "mediaType": "image/jpeg",
                "youtube:resolution": "medium"
            },
            {
                "@type": "Link",
                "href": snippet.get("thumbnails").get("high").get("url"),
                "mediaType": "image/jpeg",
                "youtube:resolution": "high"
            },

        ]
    }
    return stream_object


"""

"""


"""
{
        "@context": "http://www.w3.org/ns/activitystreams",

        "@type": "Activity", ------ Abstract wrapper

        "published": "2015-02-10T15:04:55Z",

        "provider": {
            "@type": "Service",
            "displayName": "Twitter|FaceBook|Instagram|YouTube"
        },

        "actor": {
            "@type": "Person",
            "@id": "https://www.twitter.com/{{user.screen_name}}
            "displayName": "Martin Smith",
            "url": "http://example.org/martin",
            "image": {
                "@type": "Link",
                "href": "http://example.org/martin/image.jpg",
                "mediaType": "image/jpeg"
            }
        },

    ------------------------------------------------------

        "object" : {
            "@id": "urn:example:blog:abc123/xyz",
            "@type": "Note",
            "url": "http://example.org/blog/2011/02/entry",
            "content": "This is a short note"
        },

    ------------------------------------------------------

    "object" : {
        "@id": "urn:example:blog:abc123/xyz",
        "@type": "Video",
        "displayName": "A Simple Video",
        "url": "http://example.org/video.mkv",
        "duration": "PT2H"
    },

    ------------------------------------------------------

    "object" : {
        "@id": "urn:example:blog:abc123/xyz",
        "@type": "Image",
        "displayName": "A Simple Image",
        "content": "any messages?"
        "url": [
            {
                "@type": "Link",
                "href": "http://example.org/image.jpeg",
                "mediaType": "image/jpeg"
            },

            {
                "@type": "Link",
                "href": "http://example.org/image.png",
                "mediaType": "image/png"
            }
        ]
    },
}
"""



from django.test import TransactionTestCase
from .utils import load_json
import mock
from chatterbox.models import (
    Service, Collector, Job, Activity
)


class CollectorFacebookWallTestCase(TransactionTestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def setUp(self):
        service = Service.objects.get(key='facebook')

        collector = Collector()
        collector.driver = "chatterbox.collectors.facebook.FacebookWall"
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.data = {"user_id": "11936081183"}
        job.save()
        job.keys.add(key)
        self.job = job

    def test_basic_wall_scrape(self):
        job = self.job
        # mock the request response
        return_value = load_json("facebook-brand-feed-response")

        with mock.patch('chatterbox.api.facebook.Facebook.user_feed') as mock_user_feed:
            mock_user_feed.return_value = return_value
            job.run()
            self.assertEqual(Activity.objects.all().count(), 20)

            # if we run this command at a later time, with same results, we
            # will not get any duplicate or additional Activity items added
            job.run()
            self.assertEqual(Activity.objects.all().count(), 20)



from django.test import TransactionTestCase
import mock
from .utils import (
    load_json, load_data, mock_request_with_json_response
)

from chatterbox.models import (
    Service, Collector, Job, Activity
)


class CollectorTwitterSearchTestCase(TransactionTestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def setUp(self):
        service = Service.objects.get(key='twitter')

        collector = Collector()
        collector.driver = "chatterbox.collectors.twitter.TwitterTagSearch"
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.data = {"tag": "#dino"}
        job.save()
        job.keys.add(key)
        self.job = job

    def test_basic_search(self):

        job = self.job
        # mock the request response
        return_value = load_json("twitter-basic-search-response")
        with mock.patch('chatterbox.api.twitter.Twitter.search') as mock_search:
            mock_search.return_value = return_value
            job.run()
            self.assertEqual(Activity.objects.all().count(), 2)

    def test_duplicate_results(self):
        job = self.job

        # mock the request response
        return_value = load_json("twitter-duplicate-tweets")
        with mock.patch('chatterbox.api.twitter.Twitter.search') as mock_search:
            mock_search.return_value = return_value
            job.run()
        self.assertEqual(Activity.objects.all().count(), 1)

    def test_multiple_job_tweets(self):
        job1 = self.job

        # create 2nd job
        job2 = Job()
        job2.collector = job1.collector
        job2.data = {"tag": "#dino"}
        job2.save()
        job2.keys.add(job1.keys.all()[0])

        # mock the request response
        return_value = load_json("twitter-basic-search-response")
        with mock.patch('chatterbox.api.twitter.Twitter.search') as mock_search:
            mock_search.return_value = return_value
            job1.run()
            job2.run()

        activity = Activity.objects.all()[0]
        self.assertEqual(activity.job.all().count(), 2)

        self.assertEqual(Activity.objects.all().count(), 2)

    def test_rate_limited(self):
        job1 = self.job

        return_value = load_data("twitter-rate-limit-response")
        req = mock_request_with_json_response(return_value)
        with mock.patch("requests_oauthlib.OAuth1Session.request", req):
            job1.run()

        activities = Activity.objects.all().count()
        self.assertEqual(activities, 0)



from django.test import TestCase
from chatterbox.models import(
    Service, Client, Key
)


class KeysTestCase(TestCase):

    def setUp(self):
        pass

    def test_load_api(self):
        from chatterbox.tests.api.demo import DemoApi
        service = Service()
        service.driver = "chatterbox.tests.services.demo.DemoService"

        client = Client()

        key = Key()
        key.service = service
        key.client = client

        api = key.api
        self.assertTrue(api.__class__ == DemoApi)



import mock
from django.test import TestCase
from chatterbox.models import Service
from chatterbox.exceptions import RateLimitException
from .utils import load_data, mock_request_with_json_response


class TwitterApiTestCase(TestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def setUp(self):
        self.service = Service.objects.get(key='twitter')
        self.key = self.service.service_keys.first()
        self.api = self.key.api

    def test_simple_search(self):
        data = self.api.search('flowers')

        # default count is 15
        self.assertTrue(data['search_metadata']['count'] == 15)
        self.assertTrue(data['search_metadata']['count'] == len(data['statuses']))

    def test_rate_limit(self):
        json = load_data("twitter-rate-limit-response")

        request = mock_request_with_json_response(json)
        self.api._session.request = request
        self.assertRaises(RateLimitException, self.api.search, [self, "#foo"])

    def test_complex_search(self):
        data = self.api.search('flowers', count=100)

        self.assertTrue(data['search_metadata']['count'] == 100)
        self.assertTrue(data['search_metadata']['count'] == len(data['statuses']))

    def test_get_home_timeline(self):
        data = self.api.home_timeline()
        count = len(data)
        self.assertTrue(count <= 20 and count > 0)

    def test_get_home_timeline_complex(self):
        data = self.api.home_timeline(count=50)
        count = len(data)

        self.assertTrue(count <= 50 and count > 20)



from django.test import TestCase
from chatterbox.models import(
    Collector
)


class CollectorsTestCase(TestCase):

    def setUp(self):
        pass

    def test_load_driver(self):
        from chatterbox.tests.collectors.demo import DemoCollector
        collector = Collector()
        collector.driver = "chatterbox.tests.collectors.demo.DemoCollector"

        kls = collector.load_driver()

        self.assertTrue(kls == DemoCollector)



import logging
from django import forms
from chatterbox.utils.twitter import activity_from_dict
from . import (Collector, maybe)


log = logging.getLogger(__name__)


def format_search_tag(tag):
    log.debug("Invoking format_search_tag: %s", tag)

    if not tag.startswith("#"):
        tag = "#{}".format(tag)
    return tag


class TagForm(forms.Form):
    tag = forms.CharField(label='Tag', max_length=100)
    nombre = forms.CharField(label='Nombre', max_length=100)


class TwitterTagSearch(Collector):
    form = TagForm
    activity_from_dict = activity_from_dict

    def action(self, job):
        log.debug("Invoking action")

        for status in self.statuses:
            activity = self.create_or_get_activity_from_dict(status)

            try:
                self.register_activity(activity, job)
            except Exception:
                log.debug("Stopping action")
                # need to decide when to stop...
                # at this point this job has already processed this activity
                # before, should we do nothing?  for now we are going to stop
                # all iteration and be done
                break

    @property
    def statuses(self):
        tag = format_search_tag(self.job.data["tag"])

        results = maybe(self.fetch_results)(tag)

        if results is None:
            raise StopIteration

        tweets = results.get('statuses')

        while 1:
            if len(tweets) == 0:
                raise StopIteration

            for tweet in tweets:
                yield tweet

            log.debug("Fetching next page")
            results = maybe(self.fetch_results)(tag, max_id=tweet['id_str'])

            if results is None:
                raise StopIteration

            tweets = results.get('statuses')

    def fetch_results(self, tag, **kwargs):
        return self.api.search(tag, **kwargs)

    def post_save(self, job):
        pass
        # print("GOT HERE")

    def post_delete(self, job):
        pass
        # print("GOT HERE")



import logging
from django import forms
from chatterbox.utils.facebook import activity_from_dict
from . import (Collector, maybe)


log = logging.getLogger(__name__)


class FacebookUserForm(forms.Form):
    user_id = forms.CharField(label='User ID', max_length=100)


class FacebookWall(Collector):
    form = FacebookUserForm
    activity_from_dict = activity_from_dict

    def action(self, job):
        log.debug("Invoking action")

        for status in self.statuses:
            if status.get("story") and not status.get("content", None):
                log.debug("Got unused type %s", status)
                # these are useless things like, 'dino commented on his photo'
                # we don't want this, if you do, override the collector
                continue
            activity = self.create_or_get_activity_from_dict(status)
            try:
                self.register_activity(activity, job)
            except Exception:
                log.debug("Stopping action")
                # need to decide when to stop...
                # at this point this job has already processed this activity
                # before, should we do nothing?  for now we are going to stop
                # all iteration and be done, in the future we might want to
                # keep running the process and run an update action? idk...
                break

    @property
    def statuses(self):
        user_id = self.job.data["user_id"]
        results = maybe(self.fetch_results)(user_id)

        if results is None:
            raise StopIteration

        messages = results.get('data')

        while 1:
            if len(messages) == 0:
                raise StopIteration

            for message in messages:
                yield message

            log.debug("Fetching next page")

            try:
                next_url = results["paging"]["next"]
            except KeyError:
                raise StopIteration

            # Facebook can respond with:
            # {paging: {next: "", previous: ""}}
            # so we may not get a `KeyError`, but we may
            # get an empty string, so ensure we actually
            # have a url
            if not next_url:
                raise StopIteration

            log.debug("Fetching next page")
            results = maybe(self.fetch_url)(next_url)

            if results is None:
                raise StopIteration

            messages = results.get('data')

    def fetch_results(self, user_id, **kwargs):
        return self.api.user_feed(user_id, **kwargs)

    def fetch_url(self, url, **kwargs):
        return self.api.get(url, **kwargs)



import json
import os.path
import mock


def mock_request_response():
    response = mock.Mock()
    request = mock.Mock(return_value=response)

    return request, response


def mock_json_response(response, data):
    response.content = data
    response.json = mock.Mock(return_value=json.loads(data))

    return response


def mock_request_with_json_response(data):
    request, response = mock_request_response()
    mock_json_response(response, data)

    return request


def mock_request_with_json_response_from_file(filename):
    data = load_data(filename)
    return mock_request_with_json_response(data)


def load_data(name):
    data = None

    if not name.endswith(".json"):
        name = "{}.json".format(name)

    tests_folder = os.path.dirname(os.path.realpath(__file__))
    path = os.path.join(tests_folder, "data", name)

    with open(path) as f:
        data = f.read()

    return data


def load_json(name):
    return json.loads(load_data(name))



import logging
from django.shortcuts import redirect
from django.contrib import messages
from chatterbox.models import Client, Key
from chatterbox import settings


log = logging.getLogger(__name__)


def callback(request):
    # expected_state = request.session.get('chatterbox_state', None)
    # request.session.pop('chatterbox_state', None)

    # query_state = request.GET.get('state', None)
    # import pdb; pdb.set_trace()

    # candidate_state, client_uuid = query_state.split('|')
    client_uuid = request.session.pop('client_uuid', None)
    client = None

    # code = None
    # error = request.GET.get('error', None)

    # if error:
    #     # User declined authentication
    #     # TODO Add Error Handling
    #     print("USER DECLIEND AUTH!!!!!")
    #     pass

    # if candidate_state != expected_state:
    #     # potential CSRF, bail out.
    #     # TODO Add ERROR Handling
    #     print("CSRF!!!!!! OH NO!!!")
    #     pass
    try:
        client = Client.objects.select_related("service").get(uuid=client_uuid)
    except Client.DoesNotExist as e:
        log.error('Invalid Client: %s', e)
        messages.error(request, 'OOOPS!! something went wrong, check the logs?')
        return redirect(settings.SUCCESS_REDIRECT_URL)

    # code = request.GET.get('code', None)

    # if code is None:
    #     # TODO Bail out, we didn't get a code back from the
    #     # provider, nothing we can do here.
    #     pass
    driver = client.driver
    driver.request = request

    # TODO try/except this...
    data = driver.callback(request.GET, client.redirect_url)

    key = Key.objects.create(
        access_token=data["access_token"],
        secret=data.get("secret", None),
        expires=data.get("expires_at", None),
        refresh_token=data.get("refresh_token", None),
        client=client,
        service=client.service,
        user=request.user
    )

    api = key.api
    profile = api.simple_profile()

    key.service_username = profile.name
    key.service_user_id = profile.id
    key.save()

    return redirect(settings.SUCCESS_REDIRECT_URL)



import logging
import calendar
import pytz
from datetime import (datetime, timedelta)
from requests_oauthlib import OAuth2Session, OAuth1Session


log = logging.getLogger(__name__)


class SimpleProfile(object):
    """SimpleProfile is an object whose purpose is to normalize the
    various possible responses from N services.

    Api Subclasses are required to implement the method `simple_profile`
    to return an instance of this object. Specifically, `id` and `name`
    are required to be present.
    """

    @classmethod
    def with_dict(cls, dict):
        obj = cls(**dict)
        return obj

    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.link = kwargs.get('link', None)
        self.picture = kwargs.get('picture', None)


class Api(object):

    def __init__(self, key, client):
        # meant to be overriden
        pass

    def post(self, url, **kwargs):
        log.debug("POST: %s, %s", url, kwargs)
        r = self._session.request(
            "POST",
            url=url,
            **kwargs
        )
        return r.content

    def get(self, url, **kwargs):
        log.debug("GET: %s, %s", url, kwargs)

        r = self._session.request(
            "GET",
            url=url,
            **kwargs
        )

        data = self.parse_response(r)
        self.verify_parsed_response(data)

        return data

    def verify_parsed_response(self, data):
        raise NotImplementedError("verify_response() must be defined in a subclass")

    def simple_profile(self):
        raise NotImplementedError("simple_profile() must be defined in a subclass")

    def parse_response(self, response):
        log.debug("Parsing API Response: %s", response.content)
        return response.json()

    def whoami(self):
        raise NotImplementedError("whoami() must be defined in a subclass")


class OAuth1Api(Api):
    def __init__(self, key, client):
        log.debug("Initializing OAuth1 API")
        self.key = key
        self.client = client

        self._session = OAuth1Session(
            client_key=client.client_id,
            client_secret=client.client_secret,
            resource_owner_key=key.access_token,
            resource_owner_secret=key.secret,
        )


class OAuth2Api(Api):
    def __init__(self, key, client):
        log.debug("Initializing OAuth2 API")
        self.key = key
        self.client = client

        token = {
            'access_token': key.access_token,
            'token_type': client.driver.token_type,
            'refresh_token': key.refresh_token,
        }

        if key.expires:
            expires_at = calendar.timegm(key.expires.utctimetuple())
            token['expires_at'] = expires_at

        refresh_extras = {
            "client_id": client.client_id,
            "client_secret": client.client_secret,
        }

        self._session = OAuth2Session(
            client.client_id,
            token=token,
            auto_refresh_url=client.driver.refresh_url,
            auto_refresh_kwargs=refresh_extras,
            token_updater=self.token_updater
        )

        # allowed values:
        # AUTH_HEADER = 'auth_header'
        # URI_QUERY = 'query'
        # BODY = 'body'
        # https://github.com/idan/oauthlib/blob/master/oauthlib/oauth2/rfc6749/clients/base.py#L23-L25
        self._session._client.default_token_placement = client.driver.bearer_type

    def token_updater(self, token):
        log.debug("Updating Token")

        # save access_token
        self.client.access_token = token.get('access_token')
        self.client.save()

        # save expires time
        now = datetime.now(pytz.utc)
        expires_at = now + timedelta(seconds=token.get('expires_in'))
        self.key.expires = expires_at

        # save refresh_token
        self.key.refresh_token = token.get('refresh_token')
        self.key.save()

# example
# {
# u'access_token': u'ya29.CwFYP2TJGLjyscdPS85ZofQ8C88XbxH5fP--VupiFMDWoEerHnQ',
# u'token_type': u'Bearer',
# u'expires_in': 3600,
# u'refresh_token': u'1/t2GZlVdGSO1cTGqPNC-ZX6ocAMEudVrK5jSpoR30zcRFq6',
# u'expires_at': 1422628154.377442}



from django.test import TestCase
from chatterbox.models import(
    Service
)


class ServicesTestCase(TestCase):

    def setUp(self):
        pass

    def test_load_driver(self):
        from chatterbox.tests.services.demo import DemoService
        service = Service()
        service.driver = "chatterbox.tests.services.demo.DemoService"

        kls = service.load_driver()

        self.assertTrue(kls == DemoService)



from django.test import TestCase
from chatterbox.models import Service


class YouTubeApiTestCase(TestCase):
    fixtures = ("project/apps/chatterbox/fixtures/users.json",
                "project/apps/chatterbox/fixtures/dump.json")

    def setUp(self):
        self.service = Service.objects.get(label="YouTube")
        self.key = self.service.service_keys.first()
        self.api = self.key.api

    def test_simple_search(self):
        results = self.api.search("fireworks")
        self.assertEqual(results["pageInfo"]["resultsPerPage"], 5)

    def test_complex_search(self):
        results = self.api.search("dino", maxResults=17, pageToken="CAUQAA")
        self.assertEqual(results["pageInfo"]["resultsPerPage"], 17)

    def test_get_user_videos(self):
        results = self.api.user_videos()
        self.assertLess(results["pageInfo"]["resultsPerPage"], 100)



import logging
from .oauth import OAuth2
from .oauth import BEARER_URI


log = logging.getLogger(__name__)


class Instagram(OAuth2):

    # General info about the provider
    provider_url = "http://instagram.com"
    docs_url = "http://instagram.com/developer/"
    refresh_url = None
    api_path = 'chatterbox.api.instagram.Instagram'

    # URLs to interact with the API
    authorize_url = "https://api.instagram.com/oauth/authorize/"
    access_token_url = "https://api.instagram.com/oauth/access_token"

    available_permissions = [
        (None, "read any and all data related to you"),
        ("comments", "create or delete comments"),
        ("relationships", "follow and unfollow users"),
        ("likes", "like and unlike items"),
    ]

    supports_state = False
    bearer_type = BEARER_URI



import logging
from six.moves.urllib.parse import urlencode
from . import OAuth2Api, SimpleProfile


log = logging.getLogger(__name__)


class Instagram(OAuth2Api):

    def verify_parsed_response(self, data):
        pass

    def whoami(self):
        log.debug("Invoking whoami")
        return self.get("https://api.instagram.com/v1/users/self")

    def simple_profile(self):
        log.debug("Invoking simple_profile")
        data = self.whoami()

        data = data.get('data', {})

        link = "http://instagram.com/{}".format(data.get("username", None))

        result = {
            "id": data.get("id", None),
            "name": data.get("username", None),
            "link": link,
            "picture": data.get("profile_picture", None)
        }

        profile = SimpleProfile(**result)
        return profile

    def search(self, query, **kwargs):
        """http://instagram.com/developer/endpoints/tags/

        PARAMETERS
        count -- Count of tagged media to return.
        min_tag_id -- Return media before this min_tag_id.
        max_tag_id -- Return media after this max_tag_id.

        """

        log.debug("Invoking search")

        # need to do a check on quality of query (spaces?)
        url = 'https://api.instagram.com/v1/tags/{}/media/recent'.format(query)
        if kwargs:
            additional = urlencode(kwargs)
            url = url + '?{}'.format(additional)
        return self.get(url)

    def user_media(self, user_id='self', **kwargs):
        """http://instagram.com/developer/endpoints/users/#get_users_feed

        PARAMETERS
        count -- Count of media to return.
        max_timestamp -- Return media before this UNIX timestamp.
        access_token -- A valid access token.
        min_timestamp -- Return media after this UNIX timestamp.
        min_id -- Return media later than this min_id.
        max_id -- Return media earlier than this max_id.

        """

        log.debug("Invoking user_media")
        # user_id can be None
        if not user_id:
            user_id = 'self'
        # need to do a check on quality of query (spaces?)
        url = 'https://api.instagram.com/v1/users/{}/media/recent'.format(user_id)
        if kwargs:
            additional = urlencode(kwargs)
            url = url + '?{}'.format(additional)
        return self.get(url)



import logging
import json
import uuid
import importlib
from django.db import models
from django.conf import settings
from jsonfield import JSONField
from chatterbox.utils.date import string_to_datetime
from chatterbox import settings as chatterbox_settings


log = logging.getLogger(__name__)


def make_uuid():
    return str(uuid.uuid4().hex)


def maybe_load_class(path):
    parts = path.split('.')
    module = '.'.join(parts[0:-1])
    kls = parts[-1]

    try:
        return getattr(importlib.import_module(module), kls)
    except ImportError as e:
        log.error(e)
        return None


# Twitter, Instagram, Facebook, Youtube, etc
class Service(models.Model):
    label = models.CharField(max_length=200)
    key = models.SlugField(unique=True, max_length=200, db_index=True)
    driver = models.CharField(max_length=200, unique=True, db_index=True)

    def load_driver(self, path=None):
        path = path or self.driver

        try:
            return self.__driver_class
        except AttributeError:
            self.__driver_class = maybe_load_class(path)
            return self.__driver_class

    def __unicode__(self):
        return self.label


# to get anything from a service, you need this information
# ex: FB always wants key/secret
# (instance of a service)
class Client(models.Model):
    label = models.CharField(max_length=200)
    uuid = models.CharField(max_length=36, unique=True, db_index=True,
                            default=make_uuid, editable=False)
    client_id = models.CharField(max_length=200)
    client_secret = models.CharField(max_length=200)
    redirect_url = models.URLField()

    service = models.ForeignKey('Service', related_name='keys')

    @property
    def driver(self):
        try:
            return self.__driver
        except AttributeError:
            obj = self.service.load_driver()

            self.__driver = obj(
                client_id=self.client_id,
                client_secret=self.client_secret)

            return self.__driver

    @property
    def available_permissions(self):
        driver = self.driver
        return driver.available_permissions

    def __unicode__(self):
        return self.label


class Key(models.Model):
    access_token = models.CharField(max_length=250)

    # Used in OAuth1
    secret = models.CharField(max_length=200, blank=True, null=True)

    expires = models.DateTimeField(blank=True, null=True)
    refresh_token = models.CharField(max_length=200, blank=True, null=True)
    client = models.ForeignKey('Client', related_name='client_keys')
    service = models.ForeignKey('Service', related_name='service_keys')
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             blank=True, null=True,
                             related_name="chatterbox_keys"
                             )

    service_username = models.CharField(max_length=200)
    service_user_id = models.CharField(max_length=200)

    @property
    def api(self):
        try:
            return self.__api
        except AttributeError:
            obj = self.service.load_driver()
            Api = self.load_api_class(obj.api_path)
            self.__api = Api(key=self, client=self.client)
            return self.__api

    def load_api_class(self, path=None):
        path = path or ""

        try:
            return self.__api_class
        except AttributeError:
            self.__api_class = maybe_load_class(path)
            return self.__api_class


class Collector(models.Model):
    label = models.CharField(max_length=200)  # Search Tweets
    service = models.ForeignKey('Service', related_name='collectors')
    driver = models.CharField(max_length=200, unique=True, db_index=True)

    def load_driver(self, path=None):
        path = path or self.driver

        try:
            return self.__driver_class
        except AttributeError:
            self.__driver_class = maybe_load_class(path)
            return self.__driver_class

    def __unicode__(self):
        return self.label


class Job(models.Model):
    job_id = models.CharField(max_length=36, db_index=True,
                              default=make_uuid, editable=False)
    collector = models.ForeignKey('Collector', related_name='collector_actions')
    keys = models.ManyToManyField(Key, related_name='jobs')
    data = JSONField()
    history = JSONField()

    @property
    def data_json(self):
        return json.dumps(self.data)

    @property
    def collector_instance(self):
        try:
            return self.__collector
        except AttributeError:
            kls = self.collector.load_driver()

            if kls:
                self.__collector = kls()
            else:
                self.__collector = None

            self.__collector.keys = self.keys.all()
            self.__collector.job = self

            return self.__collector

    def run(self):
        try:
            self.collector_instance.action(self)
        except AttributeError:
            # loading the collector failed and
            # collector_instance is None
            pass

    def __unicode__(self):
        return self.job_id


TYPE_CHOICES = (
    ('note', 'Note'),
    ('video', 'Video'),
    ('image', 'Image'),
)

PROVIDER_CHOICES = (
    ('facebook', 'Facebook'),
    ('instagram', 'Instagram'),
    ('youtube', 'YouTube'),
    ('twitter', 'Twitter'),
)


def default_approved():
    return chatterbox_settings.AUTO_APPROVE


class Activity(models.Model):
    is_approved = models.BooleanField(default=default_approved)
    published = models.DateTimeField(blank=True, null=True)
    object_id = models.CharField(max_length=800, unique=True)
    object_type = models.CharField(max_length=250, choices=TYPE_CHOICES)
    content = models.TextField(blank=True, null=True)
    actor_displayName = models.CharField(max_length=250, blank=True, null=True)
    actor_id = models.CharField(max_length=250, blank=True, null=True)
    job = models.ManyToManyField(Job, related_name='job_activity')
    provider_displayName = models.CharField(max_length=250,
                                            choices=PROVIDER_CHOICES)
    blob = JSONField()

    def admin_media(self):
        object_type = self.object_type
        if object_type == 'image':
            return self.get_image()
        elif object_type == 'video':
            return self.get_video()

    admin_media.allow_tags = True

    def get_image(self):
        urls = self.blob.get('object').get('url')
        if urls:
            url = urls[-1]
            href = url.get('href')
            return '<img width=200px height=200px src="{}">'.format(href)
        else:
            return ''

    def get_video(self):
        urls = self.blob.get('object').get('url')
        if urls:
            url = urls[-1]
            href = url.get('href')
        else:
            return ''
        if 'youtube' in href:
            return '<iframe src="{}?autoStart=false" frameborder="0"></iframe>'.format(href)
        return '<iframe src="{}" frameborder="0"></iframe>'.format(href)

    @classmethod
    def from_activity_dict(cls, data):
        activity = cls()
        activity.object_id = data.get('object').get('@id')
        activity.published = string_to_datetime(data.get('published'))
        activity.object_type = data.get('object').get('@type').lower()
        activity.content = data.get('object').get('content')
        activity.actor_displayName = data.get('actor').get('displayName')
        activity.actor_id = data.get('actor').get('@id')
        activity.provider_displayName = data.get('provider').get('displayName').lower()
        activity.blob = data

        return activity



from django.conf.urls import patterns, include, url
from chatterbox.views.callback import callback
from chatterbox.views.auth import authorize

urlpatterns = patterns('',
    url(r"^callback/$", callback, name="callback"),
    url(r"^authorize/(?P<uuid>[a-fA-F0-9\-]{32,36})/$", authorize, name="authorize"),
)



import logging
from six.moves.urllib.parse import urlencode
from . import OAuth1Api, SimpleProfile
from ..exceptions import RateLimitException


log = logging.getLogger(__name__)


class Twitter(OAuth1Api):

    def verify_parsed_response(self, data):
        if "errors" in data and data["errors"][0]["code"] == 88:
            log.error("Rate Limited: %s", data)
            raise RateLimitException

    def whoami(self):
        log.debug("Invoking whoami")
        return self.get("https://api.twitter.com/1.1/account/verify_credentials.json")

    def simple_profile(self):
        log.debug("Invoking simple_profile")
        data = self.whoami()

        result = {
            "id": data.get("id_str", None),
            "name": data.get("screen_name", None),
            "link": "https://twitter.com/".format(data.get("screen_name", None)),
            "picture": data.get("profile_image_url", None)
        }

        profile = SimpleProfile(**result)
        return profile

    def search(self, query, **kwargs):
        """https://dev.twitter.com/rest/reference/get/search/tweets

        Common kwargs to pass:
        geocode -- latitude,longitude,radius
        count -- Up to a maximum of 100. Defaults to 15
        until -- Returns tweets generated before the given date. Date
                 should be formatted as YYYY-MM-DD
        result_type -- Specifies what type of search results you would
                       prefer to receive. The current default is "mixed."
                       Allowed values: mixed, recent, popular
        since_id -- Returns results with an ID greater than (that is,
                    more recent than) the specified ID
        max_id -- Returns results with an ID less than (that is, older
                  than) or equal to the specified ID.
        include_entities -- The entities node will be disincluded when
                            set to false.
        callback -- If supplied, the response will use the JSONP format
                    with a callback of the given name.

        """
        log.debug("Invoking search")
        kwargs = kwargs or {}
        url = 'https://api.twitter.com/1.1/search/tweets.json?'
        kwargs['q'] = query

        return self.get(url + urlencode(kwargs))

    def home_timeline(self, **kwargs):
        """https://dev.twitter.com/rest/reference/get/statuses/home_timeline

        Common kwargs to pass:

        count -- Up to a maximum of 200. Defaults to 20
        since_id -- Returns results with an ID greater than (that is,
                    more recent than) the specified ID
        max_id -- Returns results with an ID less than (that is, older
                  than) or equal to the specified ID.
        trim_user -- When set to either true, t or 1, each tweet returned
                     in a timeline will include a user object including
                    only the status authors numerical ID. Omit this
                    parameter to receive the complete user object.
        exclude_replies -- This parameter will prevent replies from
                           appearing in the returned timeline. Using
                           exclude_replies with the count parameter will
                           mean you will receive up-to count tweets -
                           this is because the count parameter retrieves
                           that many tweets before filtering out retweets
                           and replies.

        contributor_details -- This parameter enhances the contributors
                               element of the status response to include
                               the screen_name of the contributor. By
                               default only the user_id of the contributor
                               is included.
        include_entities -- The entities node will be disincluded when
                            set to false.


        """
        log.debug("Invoking home_timeline")
        kwargs = kwargs or {}
        url = "https://api.twitter.com/1.1/statuses/home_timeline.json?"
        return self.get(url + urlencode(kwargs))



import json
from django.test import TestCase
from chatterbox.utils.instagram import activity_dict_from_dict
from .utils import load_json


class InstagramUtils(TestCase):

    def test_instagram_image_activity_dict_from_dict(self):
        data = load_json("instagram-in-basic-image")
        final = load_json("instagram-out-basic-image")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)

    def test_instagram_video_activity_dict_from_dict(self):
        data = load_json("instagram-in-basic-video")
        final = load_json("instagram-out-basic-video")
        output = activity_dict_from_dict(data)
        value1 = json.dumps(output, sort_keys=True)
        value2 = json.dumps(final, sort_keys=True)
        self.assertEqual(value1, value2)



import logging
from .oauth import OAuth2

log = logging.getLogger(__name__)


class YouTube(OAuth2):
    authorize_url = "https://accounts.google.com/o/oauth2/auth"
    access_token_url = "https://accounts.google.com/o/oauth2/token"
    refresh_url = "https://www.googleapis.com/oauth2/v3/token"
    provider_url = "http://www.youtube.com"
    docs_url = "http://www.youtube.com"
    api_path = 'chatterbox.api.youtube.YouTube'

    available_permissions = [
        ('https://www.googleapis.com/auth/userinfo.profile', 'Read your basic profile information'),
        ("https://www.googleapis.com/auth/youtube", "Manage your YouTube account"),
        ("https://www.googleapis.com/auth/youtube.readonly", "View your YouTube account"),
        ("https://www.googleapis.com/auth/youtube.upload", "Manage your YouTube videos"),
        ("https://www.googleapis.com/auth/youtubepartner", "View and manage your assets and associated content on YouTube"),
        ("https://www.googleapis.com/auth/youtubepartner-channel-audit", "View private information of your YouTube channel relevant during the audit process with a YouTube partner"),
    ]

    def get_authorize_params(self, redirect_url, scopes):
        params = super(YouTube, self).get_authorize_params(redirect_url, scopes)
        params['access_type'] = 'offline'
        params['approval_prompt'] = 'force'
        return params



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.db import migrations


def populate_collectors(apps):
    Service = apps.get_model('chatterbox', 'Service')
    Collector = apps.get_model('chatterbox', 'Collector')

    # Services
    fb = Service(label='Facebook',
                 key='facebook',
                 driver='chatterbox.drivers.facebook.Facebook')
    fb.save()

    ig = Service(label='Instagram',
                 key='instagram',
                 driver='chatterbox.drivers.instagram.Instagram')
    ig.save()

    tw = Service(label='Twitter',
                 key='twitter',
                 driver='chatterbox.drivers.twitter.Twitter')
    tw.save()

    yt = Service(label='YouTube',
                 key='youtube',
                 driver='chatterbox.drivers.youtube.YouTube')
    yt.save()

    # Collectors
    col = Collector(label='Facebook User Wall',
                    service=fb,
                    driver='chatterbox.collectors.facebook.FacebookWall')
    col.save()

    col = Collector(label='Instagram Tag Search',
                    service=ig,
                    driver='chatterbox.collectors.instagram.InstagramSearch')
    col.save()

    col = Collector(label='Instagram User Media',
                    service=ig,
                    driver='chatterbox.collectors.instagram.InstagramWall')
    col.save()

    col = Collector(label='Twitter Tag Search',
                    service=tw,
                    driver='chatterbox.collectors.twitter.TwitterTagSearch')
    col.save()

    col = Collector(label='YouTube Search',
                    service=yt,
                    driver='chatterbox.collectors.youtube.YouTubeSearch')
    col.save()

    col = Collector(label='YouTube User Videos',
                    service=yt,
                    driver='chatterbox.collectors.youtube.YouTubeUser')
    col.save()


def populate(apps, schema_editor):
    populate_collectors(apps)


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0001_initial'),
        ('chatterbox', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(populate),
    ]




# DJANGO IMPORTS
from django.conf import settings

# Admin Site Title
AUTO_APPROVE = getattr(settings, "CHATTERBOX_AUTO_APPROVE", True)
SUCCESS_REDIRECT_URL = getattr(settings,
                               "CHATTERBOX_SUCCESS_REDIRECT_URL",
                               '/admin/chatterbox/key/')



from django.test import TestCase
from chatterbox.models import Service


class InstagramApiTestCase(TestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def setUp(self):
        self.service = Service.objects.get(key='instagram')
        self.key = self.service.service_keys.first()
        self.api = self.key.api

    def test_simple_search(self):
        results = self.api.search('fireworks')
        self.assertEqual(results['meta']['code'], 200)

    def test_complex_search(self):
        results = self.api.search(
            'fireworks',
            count=17,
            min_tag_id='1423001539550109')
        self.assertEqual(len(results['data']), 17)

    def test_default_user_media(self):
        results = self.api.user_media()
        self.assertEqual(results['meta']['code'], 200)

    def test_user_media(self):
        user_id='11314839'
        results = self.api.user_media(user_id=user_id)
        self.assertEqual(results['meta']['code'], 200)
        first_post = results.get('data')[0]
        first_user_id = first_post.get('user').get('id')
        self.assertEqual(first_user_id, user_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from setuptools import setup, find_packages
import chatterbox

here = os.path.abspath(os.path.dirname(__file__))


def strip_comments(l):
    return l.strip()
    #return l.split('#', 1)[0].strip()


def reqs(*f):
    return list(filter(None, [strip_comments(l) for l in open(
        os.path.join(os.getcwd(), 'requirements', *f)).readlines()]))

install_requires = reqs('default.txt')

tests_require = []
docs_extras = reqs('docs.txt')
testing_extras = tests_require + reqs('testing.txt')

readme = open(os.path.join(here, 'README.rst')).read()
history = open(os.path.join(here, 'HISTORY.rst')).read().replace('.. :changelog:', '')

setup(
    name='django-chatterbox',
    version=chatterbox.__version__,
    description='Social listening made easy',
    url='https://github.com/blitzagency/django-chatterbox',
    long_description=readme + '\n\n' + history,
    author='Adam Venturella <aventurella@blitzagency.com>',
    author_email='aventurella@blitzagency.com',
    packages=find_packages(),
    package_dir={'chatterbox': 'chatterbox'},
    include_package_data=True,
    install_requires=install_requires,
    extras_require={
        'testing': testing_extras,
        'docs': docs_extras,
    },
    tests_require=tests_require,
    license="BSD",
    zip_safe=False,
    keywords='django social chatterbox',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3'
    ]
)



from django.test import TransactionTestCase
from .utils import load_json
import mock
from chatterbox.models import (
    Service, Collector, Job, Activity
)


class CollectorInstagrameWallTestCase(TransactionTestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/dump.json')

    def test_basic_wall_scrape(self):
        service = Service.objects.get(key='instagram')

        collector = Collector()
        collector.driver = 'chatterbox.collectors.instagram.InstagramWall'
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.save()
        job.keys.add(key)

        # mock the request response
        return_value = load_json("instagram-user-feed-response")
        with mock.patch('chatterbox.api.instagram.Instagram.user_media') as mock_user_media:
            mock_user_media.return_value = return_value
            job.run()
            self.assertEqual(Activity.objects.all().count(), 2)

    def test_user_wall_scrape(self):
        service = Service.objects.get(key='instagram')

        collector = Collector()
        collector.driver = 'chatterbox.collectors.instagram.InstagramWall'
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.save()
        job.keys.add(key)

        return_value = load_json("instagram-user-feed-response")
        with mock.patch('chatterbox.api.instagram.Instagram.user_media') as mock_user_media:
            mock_user_media.return_value = return_value
            job.data = {"user_id": "11314839"}
            job.run()
        self.assertEqual(Activity.objects.all().count(), 2)

    def test_search_scrape(self):
        service = Service.objects.get(key='instagram')

        collector = Collector()
        collector.driver = 'chatterbox.collectors.instagram.InstagramSearch'
        collector.service = service
        collector.save()

        client = service.keys.all()[0]

        key = client.client_keys.all()[0]

        job = Job()
        job.collector = collector
        job.save()
        job.keys.add(key)

        return_value = load_json("instagram-search-response")
        job.data = {"query": "fireworks"}

        with mock.patch('chatterbox.api.instagram.Instagram.search') as mock_search:
            mock_search.return_value = return_value
            job.run()

            self.assertEqual(Activity.objects.all().count(), 20)



# import the logging library
import logging
from six.moves.urllib.parse import urlencode
from . import OAuth2Api, SimpleProfile


log = logging.getLogger(__name__)
API_VERSION = "2.2"


class Facebook(OAuth2Api):

    def verify_parsed_response(self, data):
        pass

    @property
    def base_url(self):
        return "https://graph.facebook.com/v{}".format(API_VERSION)

    def whoami(self):
        log.debug("Invoking whoami")
        return self.get("https://graph.facebook.com/me")

    def simple_profile(self):
        log.debug("Invoking simple_profile")

        data = self.whoami()
        picture = 'https://graph.facebook.com/{}/picture'\
            .format(data.get('id'))
        result = {
            "id": data.get("id", None),
            "name": data.get("name", None),
            "link": data.get('link'),
            "picture": picture
        }

        profile = SimpleProfile(**result)
        return profile

    # there is no search ability on api v2+ (depricated at v1)

    def user_feed(self, user='me', **kwargs):
        """https://developers.facebook.com/docs/graph-api/reference/v2.2/user/feed
        """
        log.debug("Invoking user_feed")
        kwargs = kwargs or {}
        url = 'https://graph.facebook.com/v{}/{}/feed?'\
            .format(API_VERSION, user)
        return self.get(url + urlencode(kwargs))

    def object_detail(self, object_id):
        """https://developers.facebook.com/docs/graph-api/reference/v2.2/status
        """
        log.debug("Invoking object_detail")
        url = "{}/{}".format(self.base_url, object_id)
        return self.get(url)



import os
import logging
from chatterbox.models import Activity


log = logging.getLogger(__name__)


def activity_from_dict(data):
    log.debug("Converting Facebook dict to Activity Model")
    activity_dict = activity_dict_from_dict(data)
    return Activity.from_activity_dict(activity_dict)


def activity_dict_from_dict(blob):
    log.debug("Converting Facebook dict to activity dict: %s", blob)
    stream_object = {}

    stream_object["@context"] = "http://www.w3.org/ns/activitystreams"
    stream_object["@type"] = "Activity"
    stream_object["published"] = blob.get("created_time")
    stream_object["provider"] = {
        "@type": "Service",
        "displayName": "Facebook"
    }
    actor = blob.get("from")
    stream_object["actor"] = {
        "@type": "Person",
        "@id": "https://www.facebook.com/{}".format(actor.get("id")),
        "displayName": actor.get("name"),
        "facebook:category": actor.get("category"),
        "facebook:id": actor.get("id"),
    }

    obj_id = blob.get("object_id") or blob.get("id")
    stream_object["object"] = {
        "@id": "https://www.facebook.com/{}".format(obj_id),
        "content": blob.get("description"),
        "facebook:actions": blob.get("actions"),
        "facebook:caption": blob.get("caption"),
        "facebook:icon": blob.get("icon"),
        "facebook:id": blob.get("id"),
        # this is removed because it is never in real time, if you want likes
        # make an additional request for them using the ID of this message
        # "facebook:likes": blob.get("likes"),
        "facebook:link": blob.get("link"),
        "facebook:message": blob.get("message"),
        "facebook:message_tags": blob.get("message_tags"),
        "facebook:name": blob.get("name"),
        "facebook:picture": blob.get("picture"),
        "facebook:privacy": blob.get("privacy"),
        "facebook:to": blob.get("to"),
        "facebook:updated_time": blob.get("updated_time"),
        "facebook:shares": blob.get("shares"),
        "facebook:status_type": blob.get("status_type"),
        # this is removed because it is never in real time, if you want
        # messages make an additional request for them using
        # the ID of this message
        # "facebook:comments": blob.get("comments"),
    }
    msg_type = blob.get("type")
    if msg_type == "link":
        stream_object["object"]["@type"] = "Link"
        stream_object["object"]["href"] = blob.get("link")

    if msg_type == "photo":
        stream_object["object"]["@type"] = "Image"
        _, ext = os.path.splitext(blob.get("picture"))
        media_type = "png" if ext.startswith(".png") else "jpeg"
        stream_object["object"]["url"] = [
            {
                "@type": "Link",
                "href": blob.get("picture"),
                "mediaType": "image/{}".format(media_type),
                "facebook:resolution": "thumbnail"
            }
        ]

    if msg_type == "status":
        stream_object["object"]["@type"] = "Note"
        stream_object["object"]["content"] = blob.get("message")

    if msg_type == "video":
        stream_object["object"]["@type"] = "Video"
        stream_object["object"]["facebook:duration"] = blob.get("duration")
        stream_object["object"]["url"] = [{
            "href": blob.get("source"),
            "@type": "Link"
        }]

    return stream_object



from mock import patch
from django.test import TestCase
from chatterbox.models import(
    Collector, Key, Job
)


class JobsTestCase(TestCase):
    fixtures = ('project/apps/chatterbox/fixtures/users.json',
                'project/apps/chatterbox/fixtures/demo.json')

    def setUp(self):
        pass

    def test_run_job(self):

        with patch('chatterbox.tests.collectors.demo.DemoCollector') as MockClass:
            collector = Collector.objects.get(pk=1)
            key = Key.objects.get(pk=1)

            job = Job.objects.create(
                collector=collector,
                data={},
                history={}
            )

            job.keys.add(key)

            job.run()
            job.collector_instance.action.assert_called_once_with(job)



import logging
from chatterbox.models import Activity
from .date import timestamp_to_datetime, datetime_to_string

log = logging.getLogger(__name__)


def activity_from_dict(data):
    log.debug("Converting Instagram dict to Activity Model")
    activity_dict = activity_dict_from_dict(data)
    return Activity.from_activity_dict(activity_dict)


def activity_dict_from_dict(blob):
    log.debug("Converting Instagram dict to activity dict: %s", blob)

    stream_object = {}

    stream_object["@context"] = "http://www.w3.org/ns/activitystreams"
    stream_object["@type"] = "Activity"

    date = timestamp_to_datetime(blob.get("created_time"))
    stream_object["published"] = datetime_to_string(date)
    stream_object["provider"] = {
        "@type": "Service",
        "displayName": "Instagram"
    }

    user = blob.get("user")
    stream_object["actor"] = {
        "@type": "Person",
        "@id": "https://www.instagram.com/{}".format(user.get("username")),
        "displayName": user.get("full_name"),
        "image": {
            "@type": "Link",
            "href": user.get("profile_picture"),
            "mediaType": "image/jpeg"
        },
        "instagram:username": user.get("username"),
        "instagram:website": user.get("website"),
        "instagram:bio": user.get("bio"),
        "instagram:id": user.get("id"),
    }

    caption = blob.get("caption", {})
    images = blob.get("images")
    stream_object["object"] = {
        "@id": blob.get("link"),
        "@type": blob.get("type").title(),
        "content": caption and caption.get("text") or None,
        "url": [
            {
                "@type": "Link",
                "href": images.get("thumbnail").get("url"),
                "mediaType": "image/jpeg",
                "instagram:width": images.get("thumbnail").get("width"),
                "instagram:height": images.get("thumbnail").get("height"),
                "instagram:resolution": "thumbnail"
            },
            {
                "@type": "Link",
                "href": images.get("low_resolution").get("url"),
                "mediaType": "image/jpeg",
                "instagram:width": images.get("low_resolution").get("width"),
                "instagram:height": images.get("low_resolution").get("height"),
                "instagram:resolution": "low_resolution"
            },
            {
                "@type": "Link",
                "href": images.get("standard_resolution").get("url"),
                "mediaType": "image/jpeg",
                "instagram:width": images.get("standard_resolution").get("width"),
                "instagram:height": images.get("standard_resolution").get("height"),
                "instagram:resolution": "standard_resolution"
            }
        ],
        "instagram:user_has_liked": blob.get("user_has_liked"),
        "instagram:attribution": blob.get("attribution"),
        "instagram:tags": blob.get("tags"),
        "instagram:comments": blob.get("comments"),
        "instagram:filter": blob.get("filter"),
        "instagram:location": blob.get("location"),
        "instagram:users_in_photo": blob.get("users_in_photo"),
        "instagram:id": blob.get("id"),
        "instagram:likes": blob.get("likes"),
    }

    if blob.get("type") == "video":
        videos = blob.get("videos")
        stream_object["object"]["image"] = stream_object["object"]["url"]
        stream_object["object"]["url"] = [
            {
                "@type": "Link",
                "href": videos.get("low_bandwidth").get("url"),
                "mediaType": "video/mp4",
                "instagram:width": videos.get("low_bandwidth").get("width"),
                "instagram:height": videos.get("low_bandwidth").get("height"),
                "instagram:option": "low_bandwidth"
            },
            {
                "@type": "Link",
                "href": videos.get("low_resolution").get("url"),
                "mediaType": "video/mp4",
                "instagram:width": videos.get("low_resolution").get("width"),
                "instagram:height": videos.get("low_resolution").get("height"),
                "instagram:option": "low_resolution"
            },
            {
                "@type": "Link",
                "href": videos.get("standard_resolution").get("url"),
                "mediaType": "video/mp4",
                "instagram:width": videos.get("standard_resolution").get("width"),
                "instagram:height": videos.get("standard_resolution").get("height"),
                "instagram:option": "standard_resolution"
            },

        ]
    return stream_object



from django.test import TestCase
from chatterbox.utils.twitter import activity_dict_from_dict
from .utils import load_json


class TwitterUtils(TestCase):

    def test_activity_dict_from_dict(self):
        data = load_json("twitter-in-basic-tweet")
        final = load_json("twitter-out-basic-tweet")
        output = activity_dict_from_dict(data)
        self.assertEqual(output, final)



import logging
from chatterbox.models import Activity
from .date import twitter_date_to_datetime, datetime_to_string


log = logging.getLogger(__name__)


def activity_from_dict(data):
    log.debug("Converting Twitter dict to Activity Model")
    activity_dict = activity_dict_from_dict(data)
    return Activity.from_activity_dict(activity_dict)


def activity_dict_from_dict(blob):
    log.debug("Converting Twitter dict to activity dict: %s", blob)

    stream_object = {}
    stream_object["@context"] = "http://www.w3.org/ns/activitystreams"
    stream_object["@type"] = "Activity"
    date = twitter_date_to_datetime(blob["created_at"])

    stream_object["published"] = datetime_to_string(date)

    stream_object["provider"] = {
        "@type": "Service",
        "displayName": "Twitter"
    }
    screen_name = blob.get("user").get("screen_name")
    user = blob.get("user")
    stream_object["actor"] = {
        "@type": "Person",
        "@id": "https://www.twitter.com/{}".format(screen_name),
        "displayName": user.get("name"),
        "twitter:contributors_enabled": user.get("contributors_enabled"),
        "twitter:created_at": user.get("created_at"),
        "twitter:default_profile": user.get("default_profile"),
        "twitter:default_profile_image": user.get("default_profile_image"),
        "twitter:description": user.get("description"),
        "twitter:entities": user.get("entities"),
        "twitter:favourites_count": user.get("favourites_count"),
        "twitter:follow_request_sent": user.get("follow_request_sent"),
        "twitter:followers_count": user.get("followers_count"),
        "twitter:following": user.get("following"),
        "twitter:friends_count": user.get("friends_count"),
        "twitter:geo_enabled": user.get("geo_enabled"),
        "twitter:id": user.get("id"),
        "twitter:id_str": user.get("id_str"),
        "twitter:is_translation_enabled": user.get("is_translation_enabled"),
        "twitter:is_translator": user.get("is_translator"),
        "twitter:lang": user.get("lang"),
        "twitter:listed_count": user.get("listed_count"),
        "twitter:location": user.get("location"),
        "twitter:notifications": user.get("notifications"),
        "twitter:profile_background_color": user.get("profile_background_color"),
        "twitter:profile_background_image_url": user.get("profile_background_image_url"),
        "twitter:profile_background_image_url_https": user.get("profile_background_image_url_https"),
        "twitter:profile_background_tile": user.get("profile_background_tile"),
        "twitter:profile_banner_url": user.get("profile_banner_url"),
        "twitter:profile_image_url": user.get("profile_image_url"),
        "twitter:profile_image_url_https": user.get("profile_image_url_https"),
        "twitter:profile_link_color": user.get("profile_link_color"),
        "twitter:profile_location": user.get("profile_location"),
        "twitter:profile_sidebar_border_color": user.get("profile_sidebar_border_color"),
        "twitter:profile_sidebar_fill_color": user.get("profile_sidebar_fill_color"),
        "twitter:profile_text_color": user.get("profile_text_color"),
        "twitter:profile_use_background_image": user.get("profile_use_background_image"),
        "twitter:protected": user.get("protected"),
        "twitter:screen_name": user.get("screen_name"),
        "twitter:statuses_count": user.get("statuses_count"),
        "twitter:time_zone": user.get("time_zone"),
        "twitter:url": user.get("url"),
        "twitter:utc_offset": user.get("utc_offset"),
        "twitter:verified": user.get("verified"),
        "image": {
            "@type": "Link",
            "href": blob.get("user").get("profile_image_url"),
            "mediaType": "image/jpeg",
        },

    }

    stream_object["object"] = {
        "@id": "https://twitter.com/{}/status/{}".format(screen_name, blob.get("id")),
        "@type": "Note",
        "content": blob.get("text"),
        "twitter:entities": blob.get("entities"),
        "twitter:extended_entities": blob.get("extended_entities"),
        "twitter:contributors": blob.get("contributors"),
        "twitter:coordinates": blob.get("coordinates"),
        "twitter:favorite_count": blob.get("favorite_count"),
        "twitter:favorited": blob.get("favorited"),
        "twitter:geo": blob.get("geo"),
        "twitter:id": blob.get("id"),
        "twitter:id_str": blob.get("id_str"),
        "twitter:in_reply_to_screen_name": blob.get("in_reply_to_screen_name"),
        "twitter:in_reply_to_status_id": blob.get("in_reply_to_status_id"),
        "twitter:in_reply_to_status_id_str": blob.get("in_reply_to_status_id_str"),
        "twitter:in_reply_to_user_id": blob.get("in_reply_to_user_id"),
        "twitter:in_reply_to_user_id_str": blob.get("in_reply_to_user_id_str"),
        "twitter:lang": blob.get("lang"),
        "twitter:place": blob.get("place"),
        "twitter:possibly_sensitive": blob.get("possibly_sensitive"),
        "twitter:retweet_count": blob.get("retweet_count"),
        "twitter:retweeted": blob.get("retweeted"),
        "twitter:source": blob.get("source"),
        "twitter:text": blob.get("text"),
        "twitter:truncated": blob.get("truncated"),
    }
    return stream_object



# from .facebook import FacebookApiTestCase
# from .instagram import InstagramApiTestCase
# from .twitter import TwitterApiTestCase
# from .youtube import YouTubeApiTestCase
# from .instagram import InstagramUtils
# from .twitter import TwitterUtils
# from .facebook import FacebookUtils
# from .youtube import YoutubeUtils
